#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <devman_plugin_intf.h>
#include <devman.h>
#include <svi.h>

#define TEST_GET(_func, _value) {\
	if((err = _func(&(_value))) < 0) { \
		printf("test "#_func" err:%d\n", err);\
	} else {\
		printf(#_func" get value: %d OK\n", _value); \
	}\
}\

#define TEST_SET(_func, _value) {\
	if((err = _func(_value)) < 0) { \
		printf("test "#_func" err:%d\n", err);\
	} else {\
		printf("test "#_func" OK\n");\
	}\
}\

#define TEST_INDEX_GET(_func, _index, _value) {\
	if((err = _func(_index, &(_value))) < 0) { \
		printf("test "#_func" ,index = %d err:%d\n", _index, err);\
	} else {\
		printf(#_func" ,index = %d get value: %d OK\n", _index, _value); \
	}\
}\

#define TEST_INDEX_SET(_func, _index, _value) {\
	if((err = _func(_index, _value)) < 0) { \
		printf("test "#_func" ,index = %d err:%d\n", _index, err);\
	} else {\
		printf("test "#_func" ,index = %d OK\n", _index);\
	}\
}\

int err = 0;

void test_set_enable()
{
	int value;

	printf("set enable value (0:disable,1:enable,-1:quit):");
	if((err = scanf("%d", &value)) <= 0) {
		return;
	}

	if(value == -1)
		return;

	TEST_SET(OEM_sys_set_haptic_motor_enable, value);
}

void test_get_max_level()
{
	int max_level;

	TEST_GET(OEM_sys_get_haptic_motor_level_max, max_level);
}

int test_set_level()
{
	int level;

	while(1) {
		printf("set level (-1:quit):");
		if((err = scanf("%d", &level)) <= 0) {
			return -1;
		}

		switch(level) {
			case 0 ... 100:
				TEST_SET(OEM_sys_set_haptic_motor_level, level);
				printf("set level: %d\n", level);
				break;
			case -1:
				return 0;
			default:
				break;
		}
	}
	return 0;
}

void test_get_level()
{
	int value;
	TEST_GET(OEM_sys_get_haptic_motor_level, value);
}

void test_one_shot()
{
	int value;

	printf("set oneshot times :");
	if((err = scanf("%d", &value)) <= 0) {
		printf("scanf failed %d\n", err);
		return;
	}

	TEST_SET(OEM_sys_set_haptic_motor_oneshot, value);
}

void svi_test()
{
	int r = 0;
	int handle = 0;

	r = svi_init(&handle);
	if (r != SVI_SUCCESS){
		printf("Cannot initialize svi.\n");
	} else {
#if 0
		r = svi_play (handle, SVI_VIB_TOUCH_TOUCH, SVI_SND_TOUCH_TOUCH1);
		if (r != SVI_SUCCESS) {
			printf("Cannot play sound or vibration.\n");
		}
#endif

#if 0
		r = svi_play_sound (handle, SVI_SND_TOUCH_TOUCH1);
		if (r != SVI_SUCCESS) {
			printf("Cannot play sound.\n");
		}
#endif


#if 1
		r = svi_play_vib (handle, SVI_VIB_OPERATION_VIBRATION);
		if (r != SVI_SUCCESS) {
			printf("Cannot play vibration.\n");
		}
#endif
		// don't close handle too fast, vibrator needs time to boost
		sleep(1);

		r = svi_fini(handle);
		if (r != SVI_SUCCESS) {
			printf("Cannot close svi.\n");
		}
	}
}

void vib_test()
{
	char input,tmp;
	int r;

	while(1) {
		printf("=======================1. vib_test==========================\n");
		printf("1 : test OEM_sys_get_haptic_motor_level_max\n");
		printf("2 : test OEM_sys_get_haptic_motor_level\n");
		printf("3 : test OEM_sys_set_haptic_motor_level\n");
		printf("4 : test OEM_sys_set_haptic_motor_enable\n");
		printf("5 : test OEM_sys_set_haptic_motor_oneshot\n");
		printf("q : quit\n");
		printf("============================================================\n");

		r = scanf("%c", &input);
		if (r < 1)
			continue;

		while((tmp = getchar()) != '\n');

		switch(input) {
			case '1':
				test_get_max_level();
				break;
			case '2':
				test_get_level();
				break;
			case '3':
				test_set_level();
				break;
			case '4':
				test_set_enable();
				break;
			case '5':
				test_one_shot();
				break;
			case 'q':
				return;
			default:
				break;
		}
	}
}

void battery_test()
{
	int value;

	TEST_GET(OEM_sys_get_battery_capacity, value);
	TEST_GET(OEM_sys_get_battery_capacity_raw, value);
	TEST_GET(OEM_sys_get_battery_charge_full, value);
	TEST_GET(OEM_sys_get_battery_charge_now, value);
	TEST_GET(OEM_sys_get_battery_present, value);
	TEST_GET(OEM_sys_get_battery_health, value);
	TEST_GET(OEM_sys_get_battery_polling_required, value);
	TEST_GET(OEM_sys_get_battery_support_insuspend_charging, value);
}

void powner_test()
{
	int value;

	TEST_GET(OEM_sys_get_power_wakeup_count, value);
	TEST_SET(OEM_sys_set_power_wakeup_count, value);
	TEST_SET(OEM_sys_set_power_state, POWER_STATE_SUSPEND);
	TEST_SET(OEM_sys_set_power_state, POWER_STATE_PRE_SUSPEND);
	TEST_SET(OEM_sys_set_power_state, POWER_STATE_POST_RESUME);
}

void cpufreq_test()
{
	int value;

	TEST_GET(OEM_sys_get_cpufreq_cpuinfo_max_freq, value);
	TEST_GET(OEM_sys_get_cpufreq_cpuinfo_min_freq, value);
	TEST_GET(OEM_sys_get_cpufreq_scaling_max_freq, value);
	TEST_SET(OEM_sys_set_cpufreq_scaling_max_freq, value);
	TEST_GET(OEM_sys_get_cpufreq_scaling_min_freq, value);
	TEST_SET(OEM_sys_set_cpufreq_scaling_min_freq, value);
}

void jack_test()
{
	int value;

	TEST_GET(OEM_sys_get_jack_charger_online, value);
	TEST_GET(OEM_sys_get_jack_earjack_online, value);
	TEST_GET(OEM_sys_get_jack_earkey_online, value);
	TEST_GET(OEM_sys_get_jack_hdmi_online, value);
	TEST_GET(OEM_sys_get_jack_usb_online, value);
	TEST_GET(OEM_sys_get_jack_cradle_online, value);
	TEST_GET(OEM_sys_get_jack_tvout_online, value);
	TEST_GET(OEM_sys_get_jack_keyboard_online, value);
}

void backlight_test()
{
	int value, power_saving, index, i;

	OEM_sys_get_display_count(&index);

	for(i = 0; i < index; i++) {
		printf("================= test index = %d ==================\n", i);
		TEST_INDEX_GET(OEM_sys_get_backlight_min_brightness, i, value);
		TEST_INDEX_GET(OEM_sys_get_backlight_max_brightness, i, value);

		if((err = OEM_sys_get_backlight_brightness(i, &value, 0)) < 0) {
			printf("test OEM_sys_get_backlight_brightness power_saving = 0 err:%d\n", err);
		} else {
			printf("OEM_sys_get_backlight_brightness power_saving = 0 get value: %d OK\n", value);
		}

		if((err = OEM_sys_get_backlight_brightness(i, &value, 1)) < 0) {
			printf("test OEM_sys_get_backlight_brightness  power_saving = 1 err:%d\n", err);
		} else {
			printf("OEM_sys_get_backlight_brightness power_saving = 1 get value: %d OK\n", value);
		}
	
		if((err = OEM_sys_set_backlight_brightness(i, &value, 0)) < 0) {
			printf("test OEM_sys_set_backlight_brightness  power_saving = 0 err:%d\n", err);
		} else {
			printf("OEM_sys_set_backlight_brightness  power_saving = 0 get value: %d OK\n", value);
		}
	
		if((err = OEM_sys_set_backlight_brightness(i, &value, 1)) < 0) {
			printf("test OEM_sys_set_backlight_brightness  power_saving = 1 err:%d\n", err);
		} else {
			printf("OEM_sys_set_backlight_brightness  power_saving = 1 get value: %d OK\n", value);
		}
	
		OEM_sys_get_backlight_brightness(i, &value, 1);
	
		TEST_INDEX_SET(OEM_sys_set_backlight_dimming, i, 1);
		TEST_INDEX_GET(OEM_sys_get_backlight_acl_control, i, value);
		TEST_INDEX_SET(OEM_sys_set_backlight_acl_control, i, value)
	}
}

int main()
{
	char input,tmp;
	int r;

	OEM_sys_get_devman_plugin_interface();

	while(1) {
		printf("===================slp_device_manager_plugin test====================\n");
		printf("1 : vib_test\n");
		printf("2 : svi_test\n");
		printf("3 : battery_test\n");
		printf("4 : power_test\n");
		printf("5 : cpufreq_test\n");
		printf("6 : jack_test\n");
		printf("7 : backlight\n");
		printf("q : quit\n");
		printf("=====================================================================\n");

		r = scanf("%c", &input);
		if (r < 1)
			continue;

		while((tmp = getchar()) != '\n');

		switch(input) {
			case '1':
				vib_test();
				break;
			case '2':
				svi_test();
				break;
			case '3':
				battery_test();
				break;
			case '4':
				powner_test();
				break;
			case '5':
				cpufreq_test();
				break;
			case '6':
				jack_test();
				break;
			case '7':
				backlight_test();
				break;
			case 'q':
				return 0;
			default:
				break;
		}
	}
	return 0;
}
